from pydantic import Field
from at_common_schemas.base import BaseSchema
from datetime import datetime
from enum import Enum

class StatementPeriod(str, Enum):
    FULL_YEAR = "FULL_YEAR"
    QUARTER_1 = "QUARTER_1"
    QUARTER_2 = "QUARTER_2"
    QUARTER_3 = "QUARTER_3"
    QUARTER_4 = "QUARTER_4"

class FinancialMetadata(BaseSchema):
    symbol: str = Field(..., description="The symbol of the statement")
    date: datetime = Field(..., description="The date of the statement")
    fiscal_year: str = Field(..., description="The fiscal year of the statement")
    period: StatementPeriod = Field(..., description="The period of the statement")
    reported_currency: str = Field(..., description="The reported currency of the statement")

class FinancialStatement(FinancialMetadata):
    filing_date: datetime = Field(..., description="The filing date of the statement")
    accepted_date: datetime = Field(..., description="The accepted date of the statement")

class Income(FinancialStatement):
    revenue: float = Field(..., description="The revenue of the income statement")
    cost_of_revenue: float = Field(..., description="The cost of revenue of the income statement")
    gross_profit: float = Field(..., description="The gross profit of the income statement")
    research_and_development_expenses: float = Field(..., description="The research and development expenses of the income statement")
    general_and_administrative_expenses: float = Field(..., description="The general and administrative expenses of the income statement")
    selling_and_marketing_expenses: float = Field(..., description="The selling and marketing expenses of the income statement")
    selling_general_and_administrative_expenses: float = Field(..., description="The selling general and administrative expenses of the income statement")
    other_expenses: float = Field(..., description="The other expenses of the income statement")
    operating_expenses: float = Field(..., description="The operating expenses of the income statement")
    cost_and_expenses: float = Field(..., description="The cost and expenses of the income statement")
    net_interest_income: float = Field(..., description="The net interest income of the income statement")
    interest_income: float = Field(..., description="The interest income of the income statement")
    interest_expense: float = Field(..., description="The interest expense of the income statement")
    depreciation_and_amortization: float = Field(..., description="The depreciation and amortization of the income statement")
    ebitda: float = Field(..., description="The ebitda of the income statement")
    ebit: float = Field(..., description="The ebit of the income statement")
    non_operating_income_excluding_interest: float = Field(..., description="The non operating income excluding interest of the income statement")
    operating_income: float = Field(..., description="The operating income of the income statement")
    total_other_income_expenses_net: float = Field(..., description="The total other income expenses net of the income statement")
    income_before_tax: float = Field(..., description="The income before tax of the income statement")
    income_tax_expense: float = Field(..., description="The income tax expense of the income statement")
    net_income_from_continuing_operations: float = Field(..., description="The net income from continuing operations of the income statement")
    net_income_from_discontinued_operations: float = Field(..., description="The net income from discontinued operations of the income statement")
    other_adjustments_to_net_income: float = Field(..., description="The other adjustments to net income of the income statement")
    net_income: float = Field(..., description="The net income of the income statement")
    net_income_deductions: float = Field(..., description="The net income deductions of the income statement")
    bottom_line_net_income: float = Field(..., description="The bottom line net income of the income statement")
    eps: float = Field(..., description="The eps of the income statement")
    eps_diluted: float = Field(..., description="The eps diluted of the income statement")
    weighted_average_shs_out: float = Field(..., description="The weighted average shs out of the income statement")
    weighted_average_shs_out_diluted: float = Field(..., description="The weighted average shs out diluted of the income statement")

class BalanceSheet(FinancialStatement):
    cash_and_cash_equivalents: float = Field(..., description="The cash and cash equivalents of the balance sheet statement")
    short_term_investments: float = Field(..., description="The short term investments of the balance sheet statement")
    cash_and_short_term_investments: float = Field(..., description="The cash and short term investments of the balance sheet statement")
    net_receivables: float = Field(..., description="The net receivables of the balance sheet statement")
    accounts_receivables: float = Field(..., description="The accounts receivables of the balance sheet statement")
    other_receivables: float = Field(..., description="The other receivables of the balance sheet statement")
    inventory: float = Field(..., description="The inventory of the balance sheet statement")
    prepaids: float = Field(..., description="The prepaids of the balance sheet statement")
    other_current_assets: float = Field(..., description="The other current assets of the balance sheet statement")
    total_current_assets: float = Field(..., description="The total current assets of the balance sheet statement")
    property_plant_equipment_net: float = Field(..., description="The property plant and equipment net of the balance sheet statement")
    goodwill: float = Field(..., description="The goodwill of the balance sheet statement")
    intangible_assets: float = Field(..., description="The intangible assets of the balance sheet statement")
    goodwill_and_intangible_assets: float = Field(..., description="The goodwill and intangible assets of the balance sheet statement")
    long_term_investments: float = Field(..., description="The long term investments of the balance sheet statement")
    tax_assets: float = Field(..., description="The tax assets of the balance sheet statement")
    other_non_current_assets: float = Field(..., description="The other non current assets of the balance sheet statement")
    total_non_current_assets: float = Field(..., description="The total non current assets of the balance sheet statement")
    other_assets: float = Field(..., description="The other assets of the balance sheet statement")
    total_assets: float = Field(..., description="The total assets of the balance sheet statement")
    total_payables: float = Field(..., description="The total payables of the balance sheet statement")
    account_payables: float = Field(..., description="The account payables of the balance sheet statement")
    other_payables: float = Field(..., description="The other payables of the balance sheet statement")
    accrued_expenses: float = Field(..., description="The accrued expenses of the balance sheet statement")
    short_term_debt: float = Field(..., description="The short term debt of the balance sheet statement")
    capital_lease_obligations_current: float = Field(..., description="The capital lease obligations current of the balance sheet statement")
    tax_payables: float = Field(..., description="The tax payables of the balance sheet statement")
    deferred_revenue: float = Field(..., description="The deferred revenue of the balance sheet statement")
    other_current_liabilities: float = Field(..., description="The other current liabilities of the balance sheet statement")
    total_current_liabilities: float = Field(..., description="The total current liabilities of the balance sheet statement")
    long_term_debt: float = Field(..., description="The long term debt of the balance sheet statement")
    deferred_revenue_non_current: float = Field(..., description="The deferred revenue non current of the balance sheet statement")
    deferred_tax_liabilities_non_current: float = Field(..., description="The deferred tax liabilities non current of the balance sheet statement")
    other_non_current_liabilities: float = Field(..., description="The other non current liabilities of the balance sheet statement")
    total_non_current_liabilities: float = Field(..., description="The total non current liabilities of the balance sheet statement")
    other_liabilities: float = Field(..., description="The other liabilities of the balance sheet statement")
    capital_lease_obligations: float = Field(..., description="The capital lease obligations of the balance sheet statement")
    total_liabilities: float = Field(..., description="The total liabilities of the balance sheet statement")
    treasury_stock: float = Field(..., description="The treasury stock of the balance sheet statement")
    preferred_stock: float = Field(..., description="The preferred stock of the balance sheet statement")
    common_stock: float = Field(..., description="The common stock of the balance sheet statement")
    retained_earnings: float = Field(..., description="The retained earnings of the balance sheet statement")
    additional_paid_in_capital: float = Field(..., description="The additional paid in capital of the balance sheet statement")
    accumulated_other_comprehensive_income_loss: float = Field(..., description="The accumulated other comprehensive income loss of the balance sheet statement")
    other_total_stockholders_equity: float = Field(..., description="The other total stockholders equity of the balance sheet statement")
    total_stockholders_equity: float = Field(..., description="The total stockholders equity of the balance sheet statement")
    total_equity: float = Field(..., description="The total equity of the balance sheet statement")
    minority_interest: float = Field(..., description="The minority interest of the balance sheet statement")
    total_liabilities_and_total_equity: float = Field(..., description="The total liabilities and total equity of the balance sheet statement")
    total_investments: float = Field(..., description="The total investments of the balance sheet statement")
    total_debt: float = Field(..., description="The total debt of the balance sheet statement")
    net_debt: float = Field(..., description="The net debt of the balance sheet statement")

class CashFlow(FinancialStatement):
    net_income: float = Field(..., description="The net income of the cash flow statement")
    depreciation_and_amortization: float = Field(..., description="The depreciation and amortization of the cash flow statement")
    deferred_income_tax: float = Field(..., description="The deferred income tax of the cash flow statement")
    stock_based_compensation: float = Field(..., description="The stock based compensation of the cash flow statement")
    change_in_working_capital: float = Field(..., description="The change in working capital of the cash flow statement")
    accounts_receivables: float = Field(..., description="The accounts receivables of the cash flow statement")
    inventory: float = Field(..., description="The inventory of the cash flow statement")
    accounts_payables: float = Field(..., description="The accounts payables of the cash flow statement")
    other_working_capital: float = Field(..., description="The other working capital of the cash flow statement")
    other_non_cash_items: float = Field(..., description="The other non cash items of the cash flow statement")
    net_cash_provided_by_operating_activities: float = Field(..., description="The net cash provided by operating activities of the cash flow statement")
    investments_in_property_plant_and_equipment: float = Field(..., description="The investments in property plant and equipment of the cash flow statement")
    acquisitions_net: float = Field(..., description="The acquisitions net of the cash flow statement")
    purchases_of_investments: float = Field(..., description="The purchases of investments of the cash flow statement")
    sales_maturities_of_investments: float = Field(..., description="The sales maturities of investments of the cash flow statement")
    other_investing_activities: float = Field(..., description="The other investing activities of the cash flow statement")
    net_cash_provided_by_investing_activities: float = Field(..., description="The net cash provided by investing activities of the cash flow statement")
    net_debt_issuance: float = Field(..., description="The net debt issuance of the cash flow statement")
    long_term_net_debt_issuance: float = Field(..., description="The long term net debt issuance of the cash flow statement")
    short_term_net_debt_issuance: float = Field(..., description="The short term net debt issuance of the cash flow statement")
    net_stock_issuance: float = Field(..., description="The net stock issuance of the cash flow statement")
    net_common_stock_issuance: float = Field(..., description="The net common stock issuance of the cash flow statement")
    common_stock_issuance: float = Field(..., description="The common stock issuance of the cash flow statement")
    common_stock_repurchased: float = Field(..., description="The common stock repurchased of the cash flow statement")
    net_preferred_stock_issuance: float = Field(..., description="The net preferred stock issuance of the cash flow statement")
    net_dividends_paid: float = Field(..., description="The net dividends paid of the cash flow statement")
    common_dividends_paid: float = Field(..., description="The common dividends paid of the cash flow statement")
    preferred_dividends_paid: float = Field(..., description="The preferred dividends paid of the cash flow statement")
    other_financing_activities: float = Field(..., description="The other financing activities of the cash flow statement")
    net_cash_provided_by_financing_activities: float = Field(..., description="The net cash provided by financing activities of the cash flow statement")
    effect_of_forex_changes_on_cash: float = Field(..., description="The effect of forex changes on cash of the cash flow statement")
    net_change_in_cash: float = Field(..., description="The net change in cash of the cash flow statement")
    cash_at_end_of_period: float = Field(..., description="The cash at the end of period of the cash flow statement")
    cash_at_beginning_of_period: float = Field(..., description="The cash at the beginning of period of the cash flow statement")
    operating_cash_flow: float = Field(..., description="The operating cash flow of the cash flow statement")
    capital_expenditure: float = Field(..., description="The capital expenditure of the cash flow statement")
    free_cash_flow: float = Field(..., description="The free cash flow of the cash flow statement")
    income_taxes_paid: float = Field(..., description="The income taxes paid of the cash flow statement")
    interest_paid: float = Field(..., description="The interest paid of the cash flow statement")

class IncomeGrowth(FinancialMetadata):
    growth_revenue: float = Field(..., description="The growth revenue of the income statement growth")
    growth_cost_of_revenue: float = Field(..., description="The growth cost of revenue of the income statement growth")
    growth_gross_profit: float = Field(..., description="The growth gross profit of the income statement growth")
    growth_gross_profit_ratio: float = Field(..., description="The growth gross profit ratio of the income statement growth")
    growth_research_and_development_expenses: float = Field(..., description="The growth research and development expenses of the income statement growth")
    growth_general_and_administrative_expenses: float = Field(..., description="The growth general and administrative expenses of the income statement growth")
    growth_selling_and_marketing_expenses: float = Field(..., description="The growth selling and marketing expenses of the income statement growth")
    growth_other_expenses: float = Field(..., description="The growth other expenses of the income statement growth")
    growth_operating_expenses: float = Field(..., description="The growth operating expenses of the income statement growth")
    growth_cost_and_expenses: float = Field(..., description="The growth cost and expenses of the income statement growth")
    growth_interest_income: float = Field(..., description="The growth interest income of the income statement growth")
    growth_interest_expense: float = Field(..., description="The growth interest expense of the income statement growth")
    growth_depreciation_and_amortization: float = Field(..., description="The growth depreciation and amortization of the income statement growth")
    growth_ebitda: float = Field(..., description="The growth ebitda of the income statement growth")
    growth_operating_income: float = Field(..., description="The growth operating income of the income statement growth")
    growth_income_before_tax: float = Field(..., description="The growth income before tax of the income statement growth")
    growth_income_tax_expense: float = Field(..., description="The growth income tax expense of the income statement growth")
    growth_net_income: float = Field(..., description="The growth net income of the income statement growth")
    growth_eps: float = Field(..., description="The growth eps of the income statement growth")
    growth_eps_diluted: float = Field(..., description="The growth eps diluted of the income statement growth")
    growth_weighted_average_shs_out: float = Field(..., description="The growth weighted average shs out of the income statement growth")
    growth_weighted_average_shs_out_diluted: float = Field(..., description="The growth weighted average shs out diluted of the income statement growth")
    growth_ebit: float = Field(..., description="The growth ebit of the income statement growth")
    growth_non_operating_income_excluding_interest: float = Field(..., description="The growth non operating income excluding interest of the income statement growth")
    growth_net_interest_income: float = Field(..., description="The growth net interest income of the income statement growth")
    growth_total_other_income_expenses_net: float = Field(..., description="The growth total other income expenses net of the income statement growth")
    growth_net_income_from_continuing_operations: float = Field(..., description="The growth net income from continuing operations of the income statement growth")
    growth_other_adjustments_to_net_income: float = Field(..., description="The growth other adjustments to net income of the income statement growth")
    growth_net_income_deductions: float = Field(..., description="The growth net income deductions of the income statement growth")

class BalanceSheetGrowth(FinancialMetadata):
    growth_cash_and_cash_equivalents: float = Field(..., description="The growth cash and cash equivalents of the balance sheet statement growth")
    growth_short_term_investments: float = Field(..., description="The growth short term investments of the balance sheet statement growth")
    growth_cash_and_short_term_investments: float = Field(..., description="The growth cash and short term investments of the balance sheet statement growth")
    growth_net_receivables: float = Field(..., description="The growth net receivables of the balance sheet statement growth")
    growth_inventory: float = Field(..., description="The growth inventory of the balance sheet statement growth")
    growth_other_current_assets: float = Field(..., description="The growth other current assets of the balance sheet statement growth")
    growth_total_current_assets: float = Field(..., description="The growth total current assets of the balance sheet statement growth")
    growth_property_plant_equipment_net: float = Field(..., description="The growth property plant and equipment net of the balance sheet statement growth")
    growth_goodwill: float = Field(..., description="The growth goodwill of the balance sheet statement growth")
    growth_intangible_assets: float = Field(..., description="The growth intangible assets of the balance sheet statement growth")
    growth_goodwill_and_intangible_assets: float = Field(..., description="The growth goodwill and intangible assets of the balance sheet statement growth")
    growth_long_term_investments: float = Field(..., description="The growth long term investments of the balance sheet statement growth")
    growth_tax_assets: float = Field(..., description="The growth tax assets of the balance sheet statement growth")
    growth_other_non_current_assets: float = Field(..., description="The growth other non current assets of the balance sheet statement growth")
    growth_total_non_current_assets: float = Field(..., description="The growth total non current assets of the balance sheet statement growth")
    growth_other_assets: float = Field(..., description="The growth other assets of the balance sheet statement growth")
    growth_total_assets: float = Field(..., description="The growth total assets of the balance sheet statement growth")
    growth_account_payables: float = Field(..., description="The growth account payables of the balance sheet statement growth")
    growth_short_term_debt: float = Field(..., description="The growth short term debt of the balance sheet statement growth")
    growth_tax_payables: float = Field(..., description="The growth tax payables of the balance sheet statement growth")
    growth_deferred_revenue: float = Field(..., description="The growth deferred revenue of the balance sheet statement growth")
    growth_other_current_liabilities: float = Field(..., description="The growth other current liabilities of the balance sheet statement growth")
    growth_total_current_liabilities: float = Field(..., description="The growth total current liabilities of the balance sheet statement growth")
    growth_long_term_debt: float = Field(..., description="The growth long term debt of the balance sheet statement growth")
    growth_deferred_revenue_non_current: float = Field(..., description="The growth deferred revenue non current of the balance sheet statement growth")
    growth_deferred_tax_liabilities_non_current: float = Field(..., description="The growth deferred tax liabilities non current of the balance sheet statement growth")
    growth_other_non_current_liabilities: float = Field(..., description="The growth other non current liabilities of the balance sheet statement growth")
    growth_total_non_current_liabilities: float = Field(..., description="The growth total non current liabilities of the balance sheet statement growth")
    growth_other_liabilities: float = Field(..., description="The growth other liabilities of the balance sheet statement growth")
    growth_total_liabilities: float = Field(..., description="The growth total liabilities of the balance sheet statement growth")
    growth_preferred_stock: float = Field(..., description="The growth preferred stock of the balance sheet statement growth")
    growth_common_stock: float = Field(..., description="The growth common stock of the balance sheet statement growth")
    growth_retained_earnings: float = Field(..., description="The growth retained earnings of the balance sheet statement growth")
    growth_accumulated_other_comprehensive_income_loss: float = Field(..., description="The growth accumulated other comprehensive income loss of the balance sheet statement growth")
    growth_other_total_stockholders_equity: float = Field(..., description="The growth other total stockholders equity of the balance sheet statement growth")
    growth_total_stockholders_equity: float = Field(..., description="The growth total stockholders equity of the balance sheet statement growth")
    growth_minority_interest: float = Field(..., description="The growth minority interest of the balance sheet statement growth")
    growth_total_equity: float = Field(..., description="The growth total equity of the balance sheet statement growth")
    growth_total_liabilities_and_stockholders_equity: float = Field(..., description="The growth total liabilities and stockholders equity of the balance sheet statement growth")
    growth_total_investments: float = Field(..., description="The growth total investments of the balance sheet statement growth")
    growth_total_debt: float = Field(..., description="The growth total debt of the balance sheet statement growth")
    growth_net_debt: float = Field(..., description="The growth net debt of the balance sheet statement growth")
    growth_accounts_receivables: float = Field(..., description="The growth accounts receivables of the balance sheet statement growth")
    growth_other_receivables: float = Field(..., description="The growth other receivables of the balance sheet statement growth")
    growth_prepaids: float = Field(..., description="The growth prepaids of the balance sheet statement growth")
    growth_total_payables: float = Field(..., description="The growth total payables of the balance sheet statement growth")
    growth_other_payables: float = Field(..., description="The growth other payables of the balance sheet statement growth")
    growth_accrued_expenses: float = Field(..., description="The growth accrued expenses of the balance sheet statement growth")
    growth_capital_lease_obligations_current: float = Field(..., description="The growth capital lease obligations current of the balance sheet statement growth")
    growth_additional_paid_in_capital: float = Field(..., description="The growth additional paid in capital of the balance sheet statement growth")
    growth_treasury_stock: float = Field(..., description="The growth treasury stock of the balance sheet statement growth")

class CashFlowGrowth(FinancialMetadata):
    growth_net_income: float = Field(..., description="The growth net income of the cash flow statement growth")
    growth_net_income: float = Field(..., description="The growth net income of the cash flow statement growth")
    growth_depreciation_and_amortization: float = Field(..., description="The growth depreciation and amortization of the cash flow statement growth")
    growth_deferred_income_tax: float = Field(..., description="The growth deferred income tax of the cash flow statement growth")
    growth_stock_based_compensation: float = Field(..., description="The growth stock based compensation of the cash flow statement growth")
    growth_change_in_working_capital: float = Field(..., description="The growth change in working capital of the cash flow statement growth")
    growth_accounts_receivables: float = Field(..., description="The growth accounts receivables of the cash flow statement growth")
    growth_inventory: float = Field(..., description="The growth inventory of the cash flow statement growth")
    growth_accounts_payables: float = Field(..., description="The growth accounts payables of the cash flow statement growth")
    growth_other_working_capital: float = Field(..., description="The growth other working capital of the cash flow statement growth")
    growth_other_non_cash_items: float = Field(..., description="The growth other non cash items of the cash flow statement growth")
    growth_net_cash_provided_by_operating_activites: float = Field(..., description="The growth net cash provided by operating activites of the cash flow statement growth")
    growth_investments_in_property_plant_and_equipment: float = Field(..., description="The growth investments in property plant and equipment of the cash flow statement growth")
    growth_acquisitions_net: float = Field(..., description="The growth acquisitions net of the cash flow statement growth")
    growth_purchases_of_investments: float = Field(..., description="The growth purchases of investments of the cash flow statement growth")
    growth_sales_maturities_of_investments: float = Field(..., description="The growth sales maturities of investments of the cash flow statement growth")
    growth_other_investing_activites: float = Field(..., description="The growth other investing activites of the cash flow statement growth")
    growth_net_cash_used_for_investing_activites: float = Field(..., description="The growth net cash used for investing activites of the cash flow statement growth")
    growth_debt_repayment: float = Field(..., description="The growth debt repayment of the cash flow statement growth")
    growth_common_stock_issued: float = Field(..., description="The growth common stock issued of the cash flow statement growth")
    growth_common_stock_repurchased: float = Field(..., description="The growth common stock repurchased of the cash flow statement growth")
    growth_dividends_paid: float = Field(..., description="The growth dividends paid of the cash flow statement growth")
    growth_other_financing_activites: float = Field(..., description="The growth other financing activites of the cash flow statement growth")
    growth_net_cash_used_provided_by_financing_activities: float = Field(..., description="The growth net cash used provided by financing activities of the cash flow statement growth")
    growth_effect_of_forex_changes_on_cash: float = Field(..., description="The growth effect of forex changes on cash of the cash flow statement growth")
    growth_net_change_in_cash: float = Field(..., description="The growth net change in cash of the cash flow statement growth")
    growth_cash_at_end_of_period: float = Field(..., description="The growth cash at end of period of the cash flow statement growth")
    growth_cash_at_beginning_of_period: float = Field(..., description="The growth cash at beginning of period of the cash flow statement growth")
    growth_operating_cash_flow: float = Field(..., description="The growth operating cash flow of the cash flow statement growth")
    growth_capital_expenditure: float = Field(..., description="The growth capital expenditure of the cash flow statement growth")
    growth_free_cash_flow: float = Field(..., description="The growth free cash flow of the cash flow statement growth")
    growth_net_debt_issuance: float = Field(..., description="The growth net debt issuance of the cash flow statement growth")
    growth_long_term_net_debt_issuance: float = Field(..., description="The growth long term net debt issuance of the cash flow statement growth")
    growth_short_term_net_debt_issuance: float = Field(..., description="The growth short term net debt issuance of the cash flow statement growth")
    growth_net_stock_issuance: float = Field(..., description="The growth net stock issuance of the cash flow statement growth")
    growth_preferred_dividends_paid: float = Field(..., description="The growth preferred dividends paid of the cash flow statement growth")
    growth_income_taxes_paid: float = Field(..., description="The growth income taxes paid of the cash flow statement growth")
    growth_interest_paid: float = Field(..., description="The growth interest paid of the cash flow statement growth")

class ComprehensiveGrowth(FinancialMetadata):
    revenue_growth: float = Field(..., description="The revenue growth of the financial growth")
    gross_profit_growth: float = Field(..., description="The gross profit growth of the financial growth")
    ebit_growth: float = Field(..., description="The ebit growth of the financial growth")
    operating_income_growth: float = Field(..., description="The operating income growth of the financial growth")
    net_income_growth: float = Field(..., description="The net income growth of the financial growth")
    eps_growth: float = Field(..., description="The eps growth of the financial growth")
    eps_diluted_growth: float = Field(..., description="The eps diluted growth of the financial growth")
    weighted_average_shares_growth: float = Field(..., description="The weighted average shares growth of the financial growth")
    weighted_average_shares_diluted_growth: float = Field(..., description="The weighted average shares diluted growth of the financial growth")
    dividends_per_share_growth: float = Field(..., description="The dividends per share growth of the financial growth")
    operating_cash_flow_growth: float = Field(..., description="The operating cash flow growth of the financial growth")
    receivables_growth: float = Field(..., description="The receivables growth of the financial growth")
    inventory_growth: float = Field(..., description="The inventory growth of the financial growth")
    asset_growth: float = Field(..., description="The asset growth of the financial growth")
    book_value_per_share_growth: float = Field(..., description="The book value per share growth of the financial growth")
    debt_growth: float = Field(..., description="The debt growth of the financial growth")
    rd_expense_growth: float = Field(..., description="The rd expense growth of the financial growth")
    sga_expenses_growth: float = Field(..., description="The sga expenses growth of the financial growth")
    free_cash_flow_growth: float = Field(..., description="The free cash flow growth of the financial growth")
    ten_year_revenue_growth_per_share: float = Field(..., description="The 10 year revenue growth per share of the financial growth")
    five_year_revenue_growth_per_share: float = Field(..., description="The 5 year revenue growth per share of the financial growth")
    three_year_revenue_growth_per_share: float = Field(..., description="The 3 year revenue growth per share of the financial growth")
    ten_year_operating_cash_flow_growth_per_share: float = Field(..., description="The 10 year operating cash flow growth per share of the financial growth")
    five_year_operating_cash_flow_growth_per_share: float = Field(..., description="The 5 year operating cash flow growth per share of the financial growth")
    three_year_operating_cash_flow_growth_per_share: float = Field(..., description="The 3 year operating cash flow growth per share of the financial growth")
    ten_year_net_income_growth_per_share: float = Field(..., description="The 10 year net income growth per share of the financial growth")
    five_year_net_income_growth_per_share: float = Field(..., description="The 5 year net income growth per share of the financial growth")
    three_year_net_income_growth_per_share: float = Field(..., description="The 3 year net income growth per share of the financial growth")
    ten_year_shareholders_equity_growth_per_share: float = Field(..., description="The 10 year shareholders equity growth per share of the financial growth")
    five_year_shareholders_equity_growth_per_share: float = Field(..., description="The 5 year shareholders equity growth per share of the financial growth")
    three_year_shareholders_equity_growth_per_share: float = Field(..., description="The 3 year shareholders equity growth per share of the financial growth")
    ten_year_dividend_per_share_growth_per_share: float = Field(..., description="The 10 year dividend per share growth per share of the financial growth")
    five_year_dividend_per_share_growth_per_share: float = Field(..., description="The 5 year dividend per share growth per share of the financial growth")
    three_year_dividend_per_share_growth_per_share: float = Field(..., description="The 3 year dividend per share growth per share of the financial growth")
    ebitda_growth: float | None = Field(None, description="The ebitda growth of the financial growth")
    growth_capital_expenditure: float | None = Field(None, description="The growth capital expenditure of the financial growth")
    ten_year_bottom_line_net_income_growth_per_share: float | None = Field(None, description="The 10 year bottom line net income growth per share of the financial growth")
    five_year_bottom_line_net_income_growth_per_share: float | None = Field(None, description="The 5 year bottom line net income growth per share of the financial growth")
    three_year_bottom_line_net_income_growth_per_share: float | None = Field(None, description="The 3 year bottom line net income growth per share of the financial growth")

class Metrics(FinancialMetadata):
    market_cap: float = Field(..., description="The market cap of the key metrics")
    enterprise_value: float = Field(..., description="The enterprise value of the key metrics")
    ev_to_sales: float = Field(..., description="The ev to sales of the key metrics")
    ev_to_operating_cash_flow: float = Field(..., description="The ev to operating cash flow of the key metrics")
    ev_to_free_cash_flow: float = Field(..., description="The ev to free cash flow of the key metrics")
    ev_to_ebitda: float = Field(..., description="The ev to ebitda of the key metrics")
    net_debt_to_ebitda: float = Field(..., description="The net debt to ebitda of the key metrics")
    current_ratio: float = Field(..., description="The current ratio of the key metrics")
    income_quality: float = Field(..., description="The income quality of the key metrics")
    graham_number: float = Field(..., description="The graham number of the key metrics")
    graham_net_net: float = Field(..., description="The graham net net of the key metrics")
    tax_burden: float = Field(..., description="The tax burden of the key metrics")
    interest_burden: float = Field(..., description="The interest burden of the key metrics")
    working_capital: float = Field(..., description="The working capital of the key metrics")
    invested_capital: float = Field(..., description="The invested capital of the key metrics")
    return_on_assets: float = Field(..., description="The return on assets of the key metrics")
    operating_return_on_assets: float = Field(..., description="The operating return on assets of the key metrics")
    return_on_tangible_assets: float = Field(..., description="The return on tangible assets of the key metrics")
    return_on_equity: float = Field(..., description="The return on equity of the key metrics")
    return_on_invested_capital: float = Field(..., description="The return on invested capital of the key metrics")
    return_on_capital_employed: float = Field(..., description="The return on capital employed of the key metrics")
    earnings_yield: float = Field(..., description="The earnings yield of the key metrics")
    free_cash_flow_yield: float = Field(..., description="The free cash flow yield of the key metrics")
    capex_to_operating_cash_flow: float = Field(..., description="The capex to operating cash flow of the key metrics")
    capex_to_depreciation: float = Field(..., description="The capex to depreciation of the key metrics")
    capex_to_revenue: float = Field(..., description="The capex to revenue of the key metrics")
    sales_general_and_administrative_to_revenue: float = Field(..., description="The sales general and administrative to revenue of the key metrics")
    research_and_developement_to_revenue: float = Field(..., description="The research and development to revenue of the key metrics")
    stock_based_compensation_to_revenue: float = Field(..., description="The stock based compensation to revenue of the key metrics")
    intangibles_to_total_assets: float = Field(..., description="The intangibles to total assets of the key metrics")
    average_receivables: float = Field(..., description="The average receivables of the key metrics")
    average_payables: float = Field(..., description="The average payables of the key metrics")
    average_inventory: float = Field(..., description="The average inventory of the key metrics")
    days_of_sales_outstanding: float = Field(..., description="The days of sales outstanding of the key metrics")
    days_of_payables_outstanding: float = Field(..., description="The days of payables outstanding of the key metrics")
    days_of_inventory_outstanding: float = Field(..., description="The days of inventory outstanding of the key metrics")
    operating_cycle: float = Field(..., description="The operating cycle of the key metrics")
    cash_conversion_cycle: float = Field(..., description="The cash conversion cycle of the key metrics")
    free_cash_flow_to_equity: float = Field(..., description="The free cash flow to equity of the key metrics")
    free_cash_flow_to_firm: float = Field(..., description="The free cash flow to firm of the key metrics")
    tangible_asset_value: float = Field(..., description="The tangible asset value of the key metrics")
    net_current_asset_value: float = Field(..., description="The net current asset value of the key metrics")
    
class MetricsTTM(BaseSchema):
    symbol: str = Field(..., description="The symbol of the key metrics ttm")
    market_cap: float = Field(..., description="The market cap of the key metrics ttm")
    enterprise_value_ttm: float = Field(..., description="The enterprise value ttm of the key metrics ttm")
    ev_to_sales_ttm: float = Field(..., description="The ev to sales ttm of the key metrics ttm")
    ev_to_operating_cash_flow_ttm: float = Field(..., description="The ev to operating cash flow ttm of the key metrics ttm")
    ev_to_free_cash_flow_ttm: float = Field(..., description="The ev to free cash flow ttm of the key metrics ttm")
    ev_to_ebitda_ttm: float = Field(..., description="The ev to ebitda ttm of the key metrics ttm")
    net_debt_to_ebitda_ttm: float = Field(..., description="The net debt to ebitda ttm of the key metrics ttm")
    current_ratio_ttm: float = Field(..., description="The current ratio ttm of the key metrics ttm")
    income_quality_ttm: float = Field(..., description="The income quality ttm of the key metrics ttm")
    graham_number_ttm: float = Field(..., description="The graham number ttm of the key metrics ttm")
    graham_net_net_ttm: float = Field(..., description="The graham net net ttm of the key metrics ttm")
    tax_burden_ttm: float = Field(..., description="The tax burden ttm of the key metrics ttm")
    interest_burden_ttm: float = Field(..., description="The interest burden ttm of the key metrics ttm")
    working_capital_ttm: float = Field(..., description="The working capital ttm of the key metrics ttm")
    invested_capital_ttm: float = Field(..., description="The invested capital ttm of the key metrics ttm")
    return_on_assets_ttm: float = Field(..., description="The return on assets ttm of the key metrics ttm")
    operating_return_on_assets_ttm: float = Field(..., description="The operating return on assets ttm of the key metrics ttm")
    return_on_tangible_assets_ttm: float = Field(..., description="The return on tangible assets ttm of the key metrics ttm")
    return_on_equity_ttm: float = Field(..., description="The return on equity ttm of the key metrics ttm")
    return_on_invested_capital_ttm: float = Field(..., description="The return on invested capital ttm of the key metrics ttm")
    return_on_capital_employed_ttm: float = Field(..., description="The return on capital employed ttm of the key metrics ttm")
    earnings_yield_ttm: float = Field(..., description="The earnings yield ttm of the key metrics ttm")
    free_cash_flow_yield_ttm: float = Field(..., description="The free cash flow yield ttm of the key metrics ttm")
    capex_to_operating_cash_flow_ttm: float = Field(..., description="The capex to operating cash flow ttm of the key metrics ttm")
    capex_to_depreciation_ttm: float = Field(..., description="The capex to depreciation ttm of the key metrics ttm")
    capex_to_revenue_ttm: float = Field(..., description="The capex to revenue ttm of the key metrics ttm")
    sales_general_and_administrative_to_revenue_ttm: float = Field(..., description="The sales general and administrative to revenue ttm of the key metrics ttm")
    research_and_developement_to_revenue_ttm: float = Field(..., description="The research and development to revenue ttm of the key metrics ttm")
    stock_based_compensation_to_revenue_ttm: float = Field(..., description="The stock based compensation to revenue ttm of the key metrics ttm")
    intangibles_to_total_assets_ttm: float = Field(..., description="The intangibles to total assets ttm of the key metrics ttm")
    average_receivables_ttm: float = Field(..., description="The average receivables ttm of the key metrics ttm")
    average_payables_ttm: float = Field(..., description="The average payables ttm of the key metrics ttm")
    average_inventory_ttm: float = Field(..., description="The average inventory ttm of the key metrics ttm")
    days_of_sales_outstanding_ttm: float = Field(..., description="The days of sales outstanding ttm of the key metrics ttm")
    days_of_payables_outstanding_ttm: float = Field(..., description="The days of payables outstanding ttm of the key metrics ttm")
    days_of_inventory_outstanding_ttm: float = Field(..., description="The days of inventory outstanding ttm of the key metrics ttm")
    operating_cycle_ttm: float = Field(..., description="The operating cycle ttm of the key metrics ttm")
    cash_conversion_cycle_ttm: float = Field(..., description="The cash conversion cycle ttm of the key metrics ttm")
    free_cash_flow_to_equity_ttm: float = Field(..., description="The free cash flow to equity ttm of the key metrics ttm")
    free_cash_flow_to_firm_ttm: float = Field(..., description="The free cash flow to firm ttm of the key metrics ttm")
    tangible_asset_value_ttm: float = Field(..., description="The tangible asset value ttm of the key metrics ttm")
    net_current_asset_value_ttm: float = Field(..., description="The net current asset value ttm of the key metrics ttm")

class Ratios(FinancialMetadata):
    gross_profit_margin: float = Field(..., description="The gross profit margin of the ratios")
    ebit_margin: float = Field(..., description="The ebit margin of the ratios")
    ebitda_margin: float = Field(..., description="The ebitda margin of the ratios")
    operating_profit_margin: float = Field(..., description="The operating profit margin of the ratios")
    pretax_profit_margin: float = Field(..., description="The pretax profit margin of the ratios")
    continuous_operations_profit_margin: float = Field(..., description="The continuous operations profit margin of the ratios")
    net_profit_margin: float = Field(..., description="The net profit margin of the ratios")
    bottom_line_profit_margin: float = Field(..., description="The bottom line profit margin of the ratios")
    receivables_turnover: float = Field(..., description="The receivables turnover of the ratios")
    payables_turnover: float = Field(..., description="The payables turnover of the ratios")
    inventory_turnover: float = Field(..., description="The inventory turnover of the ratios")
    fixed_asset_turnover: float = Field(..., description="The fixed asset turnover of the ratios")
    asset_turnover: float = Field(..., description="The asset turnover of the ratios")
    current_ratio: float = Field(..., description="The current ratio of the ratios")
    quick_ratio: float = Field(..., description="The quick ratio of the ratios")
    solvency_ratio: float = Field(..., description="The solvency ratio of the ratios")
    cash_ratio: float = Field(..., description="The cash ratio of the ratios")
    price_to_earnings_ratio: float = Field(..., description="The price to earnings ratio of the ratios")
    price_to_earnings_growth_ratio: float = Field(..., description="The price to earnings growth ratio of the ratios")
    forward_price_to_earnings_growth_ratio: float = Field(..., description="The forward price to earnings growth ratio of the ratios")
    price_to_book_ratio: float = Field(..., description="The price to book ratio of the ratios")
    price_to_sales_ratio: float = Field(..., description="The price to sales ratio of the ratios")
    price_to_free_cash_flow_ratio: float = Field(..., description="The price to free cash flow ratio of the ratios")
    price_to_operating_cash_flow_ratio: float = Field(..., description="The price to operating cash flow ratio of the ratios")
    debt_to_assets_ratio: float = Field(..., description="The debt to assets ratio of the ratios")
    debt_to_equity_ratio: float = Field(..., description="The debt to equity ratio of the ratios")
    debt_to_capital_ratio: float = Field(..., description="The debt to capital ratio of the ratios")
    long_term_debt_to_capital_ratio: float = Field(..., description="The long term debt to capital ratio of the ratios")
    financial_leverage_ratio: float = Field(..., description="The financial leverage ratio of the ratios")
    working_capital_turnover_ratio: float = Field(..., description="The working capital turnover ratio of the ratios")
    operating_cash_flow_ratio: float = Field(..., description="The operating cash flow ratio of the ratios")
    operating_cash_flow_sales_ratio: float = Field(..., description="The operating cash flow sales ratio of the ratios")
    free_cash_flow_operating_cash_flow_ratio: float = Field(..., description="The free cash flow operating cash flow ratio of the ratios")
    debt_service_coverage_ratio: float = Field(..., description="The debt service coverage ratio of the ratios")
    interest_coverage_ratio: float = Field(..., description="The interest coverage ratio of the ratios")
    short_term_operating_cash_flow_coverage_ratio: float = Field(..., description="The short term operating cash flow coverage ratio of the ratios")
    operating_cash_flow_coverage_ratio: float = Field(..., description="The operating cash flow coverage ratio of the ratios")
    capital_expenditure_coverage_ratio: float = Field(..., description="The capital expenditure coverage ratio of the ratios")
    dividend_paid_and_capex_coverage_ratio: float = Field(..., description="The dividend paid and capex coverage ratio of the ratios")
    dividend_payout_ratio: float = Field(..., description="The dividend payout ratio of the ratios")
    dividend_yield: float = Field(..., description="The dividend yield of the ratios")
    dividend_yield_percentage: float = Field(..., description="The dividend yield percentage of the ratios")
    revenue_per_share: float = Field(..., description="The revenue per share of the ratios")
    net_income_per_share: float = Field(..., description="The net income per share of the ratios")
    interest_debt_per_share: float = Field(..., description="The interest debt per share of the ratios")
    cash_per_share: float = Field(..., description="The cash per share of the ratios")
    book_value_per_share: float = Field(..., description="The book value per share of the ratios")
    tangible_book_value_per_share: float = Field(..., description="The tangible book value per share of the ratios")
    shareholders_equity_per_share: float = Field(..., description="The shareholders equity per share of the ratios")
    operating_cash_flow_per_share: float = Field(..., description="The operating cash flow per share of the ratios")
    capex_per_share: float = Field(..., description="The capex per share of the ratios")
    free_cash_flow_per_share: float = Field(..., description="The free cash flow per share of the ratios")
    net_income_per_ebt: float = Field(..., description="The net income per ebt of the ratios")
    ebt_per_ebit: float = Field(..., description="The ebt per ebit of the ratios")
    price_to_fair_value: float = Field(..., description="The price to fair value of the ratios")
    debt_to_market_cap: float = Field(..., description="The debt to market cap of the ratios")
    effective_tax_rate: float = Field(..., description="The effective tax rate of the ratios")
    enterprise_value_multiple: float = Field(..., description="The enterprise value multiple of the ratios")

class RatiosTTM(BaseSchema):
    symbol: str = Field(..., description="The symbol of the ratios ttm")
    gross_profit_margin_ttm: float = Field(..., description="The gross profit margin ttm of the ratios ttm")
    ebit_margin_ttm: float = Field(..., description="The ebit margin ttm of the ratios ttm")
    ebitda_margin_ttm: float = Field(..., description="The ebitda margin ttm of the ratios ttm")
    operating_profit_margin_ttm: float = Field(..., description="The operating profit margin ttm of the ratios ttm")
    pretax_profit_margin_ttm: float = Field(..., description="The pretax profit margin ttm of the ratios ttm")
    continuous_operations_profit_margin_ttm: float = Field(..., description="The continuous operations profit margin ttm of the ratios ttm")
    net_profit_margin_ttm: float = Field(..., description="The net profit margin ttm of the ratios ttm")
    bottom_line_profit_margin_ttm: float = Field(..., description="The bottom line profit margin ttm of the ratios ttm")
    receivables_turnover_ttm: float = Field(..., description="The receivables turnover ttm of the ratios ttm")
    payables_turnover_ttm: float = Field(..., description="The payables turnover ttm of the ratios ttm")
    inventory_turnover_ttm: float = Field(..., description="The inventory turnover ttm of the ratios ttm")
    fixed_asset_turnover_ttm: float = Field(..., description="The fixed asset turnover ttm of the ratios ttm")
    asset_turnover_ttm: float = Field(..., description="The asset turnover ttm of the ratios ttm")
    current_ratio_ttm: float = Field(..., description="The current ratio ttm of the ratios ttm")
    quick_ratio_ttm: float = Field(..., description="The quick ratio ttm of the ratios ttm")
    solvency_ratio_ttm: float = Field(..., description="The solvency ratio ttm of the ratios ttm")
    cash_ratio_ttm: float = Field(..., description="The cash ratio ttm of the ratios ttm")
    price_to_earnings_ratio_ttm: float = Field(..., description="The price to earnings ratio ttm of the ratios ttm")
    price_to_earnings_growth_ratio_ttm: float = Field(..., description="The price to earnings growth ratio ttm of the ratios ttm")
    forward_price_to_earnings_growth_ratio_ttm: float = Field(..., description="The forward price to earnings growth ratio ttm of the ratios ttm")
    price_to_book_ratio_ttm: float = Field(..., description="The price to book ratio ttm of the ratios ttm")
    price_to_sales_ratio_ttm: float = Field(..., description="The price to sales ratio ttm of the ratios ttm")
    price_to_free_cash_flow_ratio_ttm: float = Field(..., description="The price to free cash flow ratio ttm of the ratios ttm")
    price_to_operating_cash_flow_ratio_ttm: float = Field(..., description="The price to operating cash flow ratio ttm of the ratios ttm")
    debt_to_assets_ratio_ttm: float = Field(..., description="The debt to assets ratio ttm of the ratios ttm")
    debt_to_equity_ratio_ttm: float = Field(..., description="The debt to equity ratio ttm of the ratios ttm")
    debt_to_capital_ratio_ttm: float = Field(..., description="The debt to capital ratio ttm of the ratios ttm")
    long_term_debt_to_capital_ratio_ttm: float = Field(..., description="The long term debt to capital ratio ttm of the ratios ttm")
    financial_leverage_ratio_ttm: float = Field(..., description="The financial leverage ratio ttm of the ratios ttm")
    working_capital_turnover_ratio_ttm: float = Field(..., description="The working capital turnover ratio ttm of the ratios ttm")
    operating_cash_flow_ratio_ttm: float = Field(..., description="The operating cash flow ratio ttm of the ratios ttm")
    operating_cash_flow_sales_ratio_ttm: float = Field(..., description="The operating cash flow sales ratio ttm of the ratios ttm")
    free_cash_flow_operating_cash_flow_ratio_ttm: float = Field(..., description="The free cash flow operating cash flow ratio ttm of the ratios ttm")
    debt_service_coverage_ratio_ttm: float = Field(..., description="The debt service coverage ratio ttm of the ratios ttm")
    interest_coverage_ratio_ttm: float = Field(..., description="The interest coverage ratio ttm of the ratios ttm")
    short_term_operating_cash_flow_coverage_ratio_ttm: float = Field(..., description="The short term operating cash flow coverage ratio ttm of the ratios ttm")
    operating_cash_flow_coverage_ratio_ttm: float = Field(..., description="The operating cash flow coverage ratio ttm of the ratios ttm")
    capital_expenditure_coverage_ratio_ttm: float = Field(..., description="The capital expenditure coverage ratio ttm of the ratios ttm")
    dividend_paid_and_capex_coverage_ratio_ttm: float = Field(..., description="The dividend paid and capex coverage ratio ttm of the ratios ttm")
    dividend_payout_ratio_ttm: float = Field(..., description="The dividend payout ratio ttm of the ratios ttm")
    dividend_yield_ttm: float = Field(..., description="The dividend yield ttm of the ratios ttm")
    enterprise_value_ttm: float = Field(..., description="The enterprise value ttm of the ratios ttm")
    revenue_per_share_ttm: float = Field(..., description="The revenue per share ttm of the ratios ttm")
    net_income_per_share_ttm: float = Field(..., description="The net income per share ttm of the ratios ttm")
    interest_debt_per_share_ttm: float = Field(..., description="The interest debt per share ttm of the ratios ttm")
    cash_per_share_ttm: float = Field(..., description="The cash per share ttm of the ratios ttm")
    book_value_per_share_ttm: float = Field(..., description="The book value per share ttm of the ratios ttm")
    tangible_book_value_per_share_ttm: float = Field(..., description="The tangible book value per share ttm of the ratios ttm")
    shareholders_equity_per_share_ttm: float = Field(..., description="The shareholders equity per share ttm of the ratios ttm")
    operating_cash_flow_per_share_ttm: float = Field(..., description="The operating cash flow per share ttm of the ratios ttm")
    capex_per_share_ttm: float = Field(..., description="The capex per share ttm of the ratios ttm")
    free_cash_flow_per_share_ttm: float = Field(..., description="The free cash flow per share ttm of the ratios ttm")
    net_income_per_ebt_ttm: float = Field(..., description="The net income per ebt ttm of the ratios ttm")
    ebt_per_ebit_ttm: float = Field(..., description="The ebt per ebit ttm of the ratios ttm")
    price_to_fair_value_ttm: float = Field(..., description="The price to fair value ttm of the ratios ttm")
    debt_to_market_cap_ttm: float = Field(..., description="The debt to market cap ttm of the ratios ttm")
    effective_tax_rate_ttm: float = Field(..., description="The effective tax rate ttm of the ratios ttm")
    enterprise_value_multiple_ttm: float = Field(..., description="The enterprise value multiple ttm of the ratios ttm")